@@ -1,5 +1,3 @@
-#pragma once
-
 /*
  * FileInfo.h, part of VCMI engine
  *
@@ -9,9 +7,12 @@
  * Full text of license available in license.txt file, in main folder
  *
  */
+#pragma once
 
 #include <boost/utility/string_ref.hpp>
 
+VCMI_LIB_NAMESPACE_BEGIN
+
 namespace FileInfo
 {
 
@@ -50,4 +51,6 @@ boost::string_ref DLL_LINKAGE GetParentPath(boost::string_ref path);
  */
 boost::string_ref DLL_LINKAGE GetPathStem(boost::string_ref path);
 
-} // namespace FileInfo
+}
+
+VCMI_LIB_NAMESPACE_END
