@@ -1,3 +1,12 @@
+/*
+ * Filesystem.cpp, part of VCMI engine
+ *
+ * Authors: listed in file AUTHORS in main folder
+ *
+ * License: GNU General Public License v2.0 or later
+ * Full text of license available in license.txt file, in main folder
+ *
+ */
 #include "StdInc.h"
 #include "Filesystem.h"
 
@@ -7,16 +16,21 @@
 #include "CZipLoader.h"
 
 //For filesystem initialization
-#include "../JsonNode.h"
 #include "../GameConstants.h"
 #include "../VCMIDirs.h"
 #include "../CStopWatch.h"
+#include "../json/JsonNode.h"
+#include "../modding/ModScope.h"
+
+VCMI_LIB_NAMESPACE_BEGIN
 
 std::map<std::string, ISimpleResourceLoader*> CResourceHandler::knownLoaders = std::map<std::string, ISimpleResourceLoader*>();
+CResourceHandler CResourceHandler::globalResourceHandler;
 
-CFilesystemGenerator::CFilesystemGenerator(std::string prefix):
+CFilesystemGenerator::CFilesystemGenerator(std::string prefix, bool extractArchives):
 	filesystem(new CFilesystemList()),
-	prefix(prefix)
+	prefix(std::move(prefix)),
+	extractArchives(extractArchives)
 {
 }
 
@@ -34,24 +48,25 @@ CFilesystemGenerator::TLoadFunctorMap CFilesystemGenerator::genFunctorMap()
 
 void CFilesystemGenerator::loadConfig(const JsonNode & config)
 {
-	for(auto & mountPoint : config.Struct())
+	for(const auto & mountPoint : config.Struct())
 	{
-		for(auto & entry : mountPoint.second.Vector())
+		for(const auto & entry : mountPoint.second.Vector())
 		{
 			CStopWatch timer;
-			logGlobal->debugStream() << "\t\tLoading resource at " << prefix + entry["path"].String();
+			logGlobal->trace("\t\tLoading resource at %s%s", prefix, entry["path"].String());
 
 			auto map = genFunctorMap();
-			auto functor = map.find(entry["type"].String());
+			auto typeName = entry["type"].String();
+			auto functor = map.find(typeName);
 
 			if (functor != map.end())
 			{
 				functor->second(mountPoint.first, entry);
-				logGlobal->debugStream() << "Resource loaded in " << timer.getDiff() << " ms.";
+				logGlobal->trace("Resource loaded in %d ms", timer.getDiff());
 			}
 			else
 			{
-				logGlobal->errorStream() << "Unknown filesystem format: " << functor->first;
+				logGlobal->error("Unknown filesystem format: %s", typeName);
 			}
 		}
 	}
@@ -67,9 +82,9 @@ void CFilesystemGenerator::loadDirectory(const std::string &mountPoint, const Js
 	std::string URI = prefix + config["path"].String();
 	int depth = 16;
 	if (!config["depth"].isNull())
-		depth = (int)config["depth"].Float();
+		depth = static_cast<int>(config["depth"].Float());
 
-	ResourceID resID(URI, EResType::DIRECTORY);
+	ResourcePath resID(URI, EResType::DIRECTORY);
 
 	for(auto & loader : CResourceHandler::get("initial")->getResourcesWithName(resID))
 	{
@@ -81,54 +96,49 @@ void CFilesystemGenerator::loadDirectory(const std::string &mountPoint, const Js
 void CFilesystemGenerator::loadZipArchive(const std::string &mountPoint, const JsonNode & config)
 {
 	std::string URI = prefix + config["path"].String();
-	auto filename = CResourceHandler::get("initial")->getResourceName(ResourceID(URI, EResType::ARCHIVE_ZIP));
+	auto filename = CResourceHandler::get("initial")->getResourceName(ResourcePath(URI, EResType::ARCHIVE_ZIP));
 	if (filename)
 		filesystem->addLoader(new CZipLoader(mountPoint, *filename), false);
 }
 
-template<EResType::Type archiveType>
+template<EResType archiveType>
 void CFilesystemGenerator::loadArchive(const std::string &mountPoint, const JsonNode & config)
 {
 	std::string URI = prefix + config["path"].String();
-	auto filename = CResourceHandler::get("initial")->getResourceName(ResourceID(URI, archiveType));
+	auto filename = CResourceHandler::get("initial")->getResourceName(ResourcePath(URI, archiveType));
 	if (filename)
-		filesystem->addLoader(new CArchiveLoader(mountPoint, *filename), false);
+		filesystem->addLoader(new CArchiveLoader(mountPoint, *filename, extractArchives), false);
 }
 
 void CFilesystemGenerator::loadJsonMap(const std::string &mountPoint, const JsonNode & config)
 {
 	std::string URI = prefix + config["path"].String();
-	auto filename = CResourceHandler::get("initial")->getResourceName(ResourceID(URI, EResType::TEXT));
+	auto filename = CResourceHandler::get("initial")->getResourceName(JsonPath::builtin(URI));
 	if (filename)
 	{
-		auto configData = CResourceHandler::get("initial")->load(ResourceID(URI, EResType::TEXT))->readAll();
-		const JsonNode config((char*)configData.first.get(), configData.second);
-		filesystem->addLoader(new CMappedFileLoader(mountPoint, config), false);
+		auto configData = CResourceHandler::get("initial")->load(JsonPath::builtin(URI))->readAll();
+		const JsonNode configInitial(reinterpret_cast<std::byte *>(configData.first.get()), configData.second, URI);
+		filesystem->addLoader(new CMappedFileLoader(mountPoint, configInitial), false);
 	}
 }
 
-void CResourceHandler::clear()
-{
-	delete knownLoaders["root"];
-}
-
 ISimpleResourceLoader * CResourceHandler::createInitial()
 {
 	//temporary filesystem that will be used to initialize main one.
 	//used to solve several case-sensivity issues like Mp3 vs MP3
-	auto initialLoader = new CFilesystemList;
+	auto * initialLoader = new CFilesystemList();
 
 	//recurse only into specific directories
-	auto recurseInDir = [&](std::string URI, int depth)
+	auto recurseInDir = [&](const std::string & URI, int depth)
 	{
-		ResourceID ID(URI, EResType::DIRECTORY);
+		ResourcePath ID(URI, EResType::DIRECTORY);
 
 		for(auto & loader : initialLoader->getResourcesWithName(ID))
 		{
 			auto filename = loader->getResourceName(ID);
 			if (filename)
 			{
-				auto dir = new CFilesystemLoader(URI + '/', *filename, depth, true);
+				auto * dir = new CFilesystemLoader(URI + '/', *filename, depth, true);
 				initialLoader->addLoader(dir, false);
 			}
 		}
@@ -137,7 +147,7 @@ ISimpleResourceLoader * CResourceHandler::createInitial()
 	for (auto & path : VCMIDirs::get().dataPaths())
 	{
 		if (boost::filesystem::is_directory(path)) // some of system-provided paths may not exist
-			initialLoader->addLoader(new CFilesystemLoader("", path, 0, true), false);
+			initialLoader->addLoader(new CFilesystemLoader("", path, 1, true), false);
 	}
 	initialLoader->addLoader(new CFilesystemLoader("", VCMIDirs::get().userDataPath(), 0, true), false);
 
@@ -150,7 +160,7 @@ ISimpleResourceLoader * CResourceHandler::createInitial()
 
 void CResourceHandler::initialize()
 {
-	// Create tree-loke structure that looks like this:
+	// Create tree-like structure that looks like this:
 	// root
 	// |
 	// |- initial
@@ -164,11 +174,16 @@ void CResourceHandler::initialize()
 	//    |-saves
 	//    |-config
 
-	knownLoaders["root"] = new CFilesystemList();
+	// when built as single process, server can be started multiple times
+	if (globalResourceHandler.rootLoader)
+		return;
+
+	globalResourceHandler.rootLoader = std::make_unique<CFilesystemList>();
+	knownLoaders["root"] = globalResourceHandler.rootLoader.get();
 	knownLoaders["saves"] = new CFilesystemLoader("SAVES/", VCMIDirs::get().userSavePath());
 	knownLoaders["config"] = new CFilesystemLoader("CONFIG/", VCMIDirs::get().userConfigPath());
 
-	auto localFS = new CFilesystemList();
+	auto * localFS = new CFilesystemList();
 	localFS->addLoader(knownLoaders["saves"], true);
 	localFS->addLoader(knownLoaders["config"], true);
 
@@ -177,38 +192,74 @@ void CResourceHandler::initialize()
 	addFilesystem("root", "local", localFS);
 }
 
+void CResourceHandler::destroy()
+{
+	knownLoaders.clear();
+	globalResourceHandler.rootLoader.reset();
+}
+
 ISimpleResourceLoader * CResourceHandler::get()
 {
 	return get("root");
 }
 
-ISimpleResourceLoader * CResourceHandler::get(std::string identifier)
+ISimpleResourceLoader * CResourceHandler::get(const std::string & identifier)
 {
+	assert(knownLoaders.count(identifier));
 	return knownLoaders.at(identifier);
 }
 
-void CResourceHandler::load(const std::string &fsConfigURI)
+void CResourceHandler::load(const std::string &fsConfigURI, bool extractArchives)
 {
-	auto fsConfigData = get("initial")->load(ResourceID(fsConfigURI, EResType::TEXT))->readAll();
+	auto fsConfigData = get("initial")->load(JsonPath::builtin(fsConfigURI))->readAll();
 
-	const JsonNode fsConfig((char*)fsConfigData.first.get(), fsConfigData.second);
+	const JsonNode fsConfig(reinterpret_cast<std::byte *>(fsConfigData.first.get()), fsConfigData.second, fsConfigURI);
 
-	addFilesystem("data", "core", createFileSystem("", fsConfig["filesystem"]));
+	addFilesystem("data", ModScope::scopeBuiltin(), createFileSystem("", fsConfig["filesystem"], extractArchives));
 }
 
 void CResourceHandler::addFilesystem(const std::string & parent, const std::string & identifier, ISimpleResourceLoader * loader)
 {
-	assert(knownLoaders.count(identifier) == 0);
+	if(knownLoaders.count(identifier) != 0)
+	{
+		logMod->error("[CRITICAL] Virtual filesystem %s already loaded!", identifier);
+		delete loader;
+		return;
+	}
 
-	auto list = dynamic_cast<CFilesystemList *>(knownLoaders.at(parent));
+	if(knownLoaders.count(parent) == 0)
+	{
+		logMod->error("[CRITICAL] Parent virtual filesystem %s for %s not found!", parent, identifier);
+		delete loader;
+		return;
+	}
+
+	auto * list = dynamic_cast<CFilesystemList *>(knownLoaders.at(parent));
 	assert(list);
 	list->addLoader(loader, false);
 	knownLoaders[identifier] = loader;
 }
 
-ISimpleResourceLoader * CResourceHandler::createFileSystem(const std::string & prefix, const JsonNode &fsConfig)
+bool CResourceHandler::removeFilesystem(const std::string & parent, const std::string & identifier)
 {
-	CFilesystemGenerator generator(prefix);
+	if(knownLoaders.count(identifier) == 0)
+		return false;
+	
+	if(knownLoaders.count(parent) == 0)
+		return false;
+
+	auto * list = dynamic_cast<CFilesystemList *>(knownLoaders.at(parent));
+	assert(list);
+	list->removeLoader(knownLoaders[identifier]);
+	knownLoaders.erase(identifier);
+	return true;
+}
+
+ISimpleResourceLoader * CResourceHandler::createFileSystem(const std::string & prefix, const JsonNode &fsConfig, bool extractArchives)
+{
+	CFilesystemGenerator generator(prefix, extractArchives);
 	generator.loadConfig(fsConfig);
 	return generator.getFilesystem();
 }
+
+VCMI_LIB_NAMESPACE_END
